// AWS SDKからBedrockのクライアントとコマンドをインポート
const { InvokeModelCommand } = require("@aws-sdk/client-bedrock-runtime");
const client = require('./client');

/**
 * コードをClaudeモデルで分析する関数
 * @param {string} code - 分析対象のコード（VBAまたはHTML）
 * @param {string} promptContent - 使用するプロンプトの内容
 * @returns {Promise<string>} - 改善されたコード
 */
async function analyzeCode(code, promptContent) {
    try {
        // 入力検証
        if (!code || !promptContent) {
            throw new Error('コードとプロンプトの内容は必須です');
        }

        // プロンプト内容にコードを挿入
        // {code}をユーザー入力のコードに置換
        const fullPrompt = promptContent.replace('{code}', code);

        // Claude用のリクエストコマンドを作成
        const command = new InvokeModelCommand({
            // Claude 3 Sonnetモデルを指定（最新バージョン）
            modelId: "anthropic.claude-3-sonnet-20240229-v1:0",
            contentType: "application/json",
            accept: "application/json",
            // Bedrock API用のリクエストボディを構築
            body: JSON.stringify({
                // Bedrockのバージョンを指定
                anthropic_version: "bedrock-2023-05-31",
                // 最大トークン数（出力の長さ制限）
                max_tokens: 2048,
                // メッセージ形式でプロンプトを送信
                messages: [
                    {
                        role: "user",
                        content: fullPrompt
                    }
                ],
                // 必要に応じて追加のパラメータを設定可能
                // temperature: 0.7,  // 生成の多様性（0-1）
                // top_p: 0.9,       // 出力のランダム性
            })
        });

        // BedrockAPIを呼び出し
        const response = await client.send(command);
        
        // レスポンスの処理（バイナリからUTF-8文字列に変換）
        const responseBody = Buffer.from(response.body).toString('utf8');
        const jsonResponse = JSON.parse(responseBody);

        // Claudeのレスポンス形式から結果を抽出
        // content[0].textに実際の生成テキストが含まれる
        if (jsonResponse.content && jsonResponse.content[0]) {
            return jsonResponse.content[0].text;
        }

        throw new Error('AIモデルからの応答が不正な形式です');

    } catch (error) {
        // エラーログを出力し、上位層に伝播
        console.error("コード分析エラー:", error);
        console.error("エラー詳細:", {
            message: error.message,
            code: error.code,
            requestId: error.$metadata?.requestId
        });
        throw error;
    }
}

// analyzeCode関数をエクスポート
module.exports = {
    analyzeCode
};