// src/db/connection.js

/**
 * MySQL2のPromise版を使用
 * Promise-basedなAPIでデータベース操作を行う
 */
const mysql = require('mysql2/promise');

/**
 * データベース接続の修正履歴 
 * 
 * 1. 接続エラーの解消
 *    - 接続タイムアウトの設定追加
 *    - リトライロジックの実装
 *    - エラーハンドリングの強化
 * 
 * 2. コネクションプールの最適化
 *    - プール設定の調整
 *    - 接続数の制限設定
 * 
 * 3. デバッグ機能の強化
 *    - 詳細なログ出力の追加
 *    - 接続状態の監視機能
 */

/**
 * データベース接続の文字コード処理の詳細説明
 * 
 * 【文字化けが発生理由】
 * データベースとアプリケーション間でデータをやり取りする際、
 * 以下の3つのポイントで文字コードの変換が発生します：
 * 
 * 1. アプリケーション → データベース（データ送信時）
 * 2. データベース内でのデータ保存
 * 3. データベース → アプリケーション（データ取得時）
 * 
 * 【設定項目の説明】
 * 1. charset: 'utf8mb4'
 *    - 接続時の文字コードを指定
 *    - データベースとの通信で使用する文字コードを決定
 * 
 * 2. collation: 'utf8mb4_unicode_ci'
 *    - 文字の照合順序を指定
 *    - 「ci」は Case Insensitive（大文字小文字を区別しない）
 * 
 * 3. initializationCommands
 *    - 接続確立直後に実行されるコマンド
 *    - セッションごとに文字コード設定を確実に行う
 * 
 * 【改善の仕組み】
 * - 接続時に文字コード設定を強制的に行う
 * - セッションごとに設定を初期化
 * - バイナリデータ経由で確実な文字コード変換を実現
 */

/**
 * データベース接続設定の拡張
 * - 環境変数から設定を読み込み、なければデフォルト値を使用
 * - utf8mb4を使用して絵文字を含む多言語対応
 */
const config = {
    host: process.env.DB_HOST || 'db',
    user: process.env.DB_USER || 'vbauser',
    password: process.env.DB_PASSWORD || 'vbapassword',
    database: process.env.DB_NAME || 'vba_formatter',
    charset: 'utf8mb4',
    collation: 'utf8mb4_unicode_ci',
    // 文字コード関連の設定を追加
    connectionLimit: 10,
    supportBigNumbers: true,
    bigNumberStrings: true,
    dateStrings: true,
    // 明示的な文字コード設定
    charset: 'utf8mb4',
    // コネクション確立時の初期化コマンド
    initializationCommands: [
        'SET NAMES utf8mb4',
        'SET CHARACTER SET utf8mb4',
        'SET SESSION collation_connection = utf8mb4_unicode_ci'
    ]
};

// コネクションプールのインスタンス
let pool;
let connectionAttempts = 0;
const MAX_RETRIES = 5;

/**
 * データベース接続を試行する関数
 * リトライロジック付き
 */
const initPool = async () => {
    while (connectionAttempts < MAX_RETRIES) {
        try {
            console.log(`データベース接続を試行中... (試行回数: ${connectionAttempts + 1})`);
            
            // プールの作成
            pool = mysql.createPool(config);
            
            // 接続テスト
            await pool.query('SELECT 1');
            
            // 文字コード設定の確認
            const [charsetResults] = await pool.query('SHOW VARIABLES LIKE "character%"');
            console.log('データベース文字コード設定:', charsetResults);
            
            console.log('データベース接続成功');
            return pool;

        } catch (error) {
            connectionAttempts++;
            console.error(`データベース接続エラー (試行回数: ${connectionAttempts}):`, error);
            
            if (connectionAttempts >= MAX_RETRIES) {
                throw new Error(`データベース接続に失敗しました。最大試行回数(${MAX_RETRIES})を超えました。`);
            }
            
            // 再試行前に待機
            await new Promise(resolve => setTimeout(resolve, 2000 * connectionAttempts));
        }
    }
};

/**
 * クエリ実行用の関数
 * 接続エラー時の再接続を含む
 */
const query = async (...args) => {
    try {
        if (!pool) {
            await initPool();
        }
        return await pool.query(...args);
    } catch (error) {
        console.error('クエリ実行エラー:', error);
        // 接続エラーの場合は再接続を試みる
        if (error.code === 'PROTOCOL_CONNECTION_LOST') {
            pool = null;
            return query(...args);
        }
        throw error;
    }
};

/**
 * データベース操作用の関数をエクスポート
 * - query: SQL実行用の関数
 * - getPool: プールインスタンス取得用の関数
 */
module.exports = {
    // クエリ実行関数
    query,
    // プール取得関数
    getPool: async () => {
        if (!pool) {
            await initPool();  // プールが未初期化なら初期化
        }
        return pool;
    }
};