/**
 * 文字コード設定の詳細説明
 * 
 * 【文字化け発生の背景】
 * データベースでは文字データを保存する際に「文字コード」という形式を使用します。
 * 日本語などの多言語文字を正しく扱うにはUTF-8mb4という文字コードが必要です。
 * 
 * 【各設定の役割】
 * 1. SET NAMES utf8mb4
 *    - クライアントとサーバー間の通信で使用する文字コードを設定
 *    - データの送受信時の文字化けを防ぐ
 * 
 * 2. SET CHARACTER SET utf8mb4
 *    - データベースが使用する文字コードを設定
 *    - データ保存時の文字化けを防ぐ
 * 
 * 3. COLLATE utf8mb4_unicode_ci
 *    - 文字の照合順序（ソート順）を設定
 *    - 日本語を含む多言語での正しい並び順を保証
 * 
 * 【utf8mb4を使用する理由】
 * - 絵文字を含むすべてのUnicode文字を扱える
 * - 従来のutf8より広い文字範囲をサポート
 * - 将来的な文字コードの拡張にも対応可能
 */

-- データベースの文字コード設定を確実に行う
SET NAMES utf8mb4;
SET CHARACTER SET utf8mb4;

-- データベースの文字コードを強制的にUTF8mb4に設定
-- これによりデータベースレベルで日本語を正しく扱える
ALTER DATABASE vba_formatter CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;

-- カラム
CREATE TABLE IF NOT EXISTS prompts (
    id VARCHAR(50) PRIMARY KEY,
    name VARCHAR(100) NOT NULL,
    content TEXT NOT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
) ENGINE=InnoDB 
  CHARACTER SET utf8mb4 
  COLLATE utf8mb4_unicode_ci;

/**
 * プロンプトの表示順序の制御について
 * 
 * 【表示順序の制御方法】
 * 1. nameカラムに接頭辞を付けて制御
 *    - 数字の場合はゼロ埋めして2桁で統一
 *    - 例：'01_', '02_' など
 * 
 * 2. 意図した順序：
 *    - "01_VBAプロンプト1" （最初に表示）
 *    - "02_VBAプロンプト2" （2番目に表示）
 *    - "03_HTML BEMの命名規則" （3番目に表示）
 *    - "04_開発テスト用" （4番目に表示）
 */

-- 初期データの挿入（順序を制御するため、nameを修正）
INSERT INTO prompts (id, name, content) VALUES 
    ('prompt01', '01_VBAプロンプト1', 
    '# ここからがAIへの指示内容です
=============================================

# AIの役割定義
あなたはVBAコードの専門家として、以下のコードを分析し、優先順位に従って改善してください。

# 入力コード部分
```vba
{code}
```

# 改善要件（優先順位：高 - 第一弾）
=============================================
1. インデント処理
  - SUBからENDSUBまでTABひとつ分のインデント
  - IF文、With文、For文等の制御構造にもインデント
  - ネストレベルに応じて適切なインデント

2. ヘッダー・フッター追加
  - Sub開始時のヘッダー:
    Option Explicit
    ''***************************************************
    '' [プロシージャ名]
    ''***************************************************
    ''【機　　能】[機能の説明]
    ''【引　　数】[引数の説明]
    ''【戻 り 値】[戻り値の説明]
    ''【機能説明】[詳細な説明]
    ''【備　　考】[その他特記事項]
    '' Copyright(c) 2024 Your Company All Rights Reserved.
    ''***************************************************
    
  - 処理の区切り:
    ''***************************************************
    '' 変数宣言
    ''***************************************************
    [変数宣言部分]

  - Sub終了時のフッター:
    ''***************************************************
    ''---------------------------------------------------
    '' Version
    ''---------------------------------------------------
    '' 1.00 | 2024.01.01 | ********* | *********
    ''***************************************************

# 改善要件（優先順位：中 - 第二弾）
=============================================
3. 変数名の改善
  - 一文字の変数を禁止
  - セル参照の変数は意味のある名前に変更
    （例: Last_Row, Last_Column）
  - 配列はArrayを付ける
  - その他の変数はDataを付ける

4. Public変数・Call文の処理
  - Public変数にはPublic関数であることをコメントで記載
  - Call文にはモジュール名を追加
    （例: Call Module1.印刷）

5. 変数宣言のコメント
  - 宣言した変数の横にコメントを追加
  - 使用目的や内容を簡潔に説明

# 改善要件（優先順位：中 - 第三弾）
=============================================
6. 変数・シート名の処理
  - 日本語変数名を英語に変更
  - 未宣言変数を「変数宣言」セクションに追加
  - 型判定できない変数はVariantに
  - シート名は定数（Const）で定義
  - シート追加時の名前はVariantで処理

7. コメント追加
  - IF文、With文などの制御構造にコメント
  - 配列の内容説明
  - SET文の説明
  - Offsetのコメント必須
    （参照セルと目的を明記）

8. コードの最適化
  - 類似コードが3回以上続く場合はループ化
  - パスの直書きは避け、pathに置換
    （元のパスはコメントとして保持）

# 出力形式
=============================================
優先順位の高い要件（第一弾）から順に適用し、
改善したVBAコードのみを出力してください。'),

    ('prompt02', '02_VBAプロンプト2',
    '# ここからがAIへの指示内容です（コードレビュー版）
=============================================

# AIの役割定義
あなたはシニアVBA開発者として、以下のコードをレビューし、ベストプラクティスに基づいて改善してください。

# 入力コード部分
```vba
{code}
```

# レビュー観点
=============================================
1. コーディング規約準拠
2. バグの可能性
3. パフォーマンスボトルネック
4. セキュリティリスク

# 出力形式
=============================================
コードレビューコメントと改善後のコードを提供してください。'),

    ('prompt03', '03_HTML BEMの命名規則',
    '# ここからがAIへの指示内容です（HTML BEM分析版）
=============================================

# AIの役割定義
あなたはHTMLとCSSの専門家として、クラス名をBEM命名規則に基づいて改善します。

# 入力コード部分
```html
{code}
```

# 改善要件
=============================================
1. BEMの基本ルール適用
  - Block: 独立したコンポーネント（例: header, menu）
  - Element: Blockの一部（例: menu__item）
  - Modifier: 状態や見た目の変更（例: menu__item--active）

2. 命名規則の統一
  - Blockは意味のある名前を使用
  - Element は __ (アンダースコア2つ) で接続
  - Modifier は -- (ハイフン2つ) で接続
  - 全て小文字、ハイフンで単語を区切る

3. クラス名の階層構造
  - 最大2階層までの要素の入れ子
  - Block内のElement同士の依存関係を避ける
  - 共通の機能はMixinとして抽出

4. コンポーネントの分割
  - 再利用可能なBlockの特定
  - 共通のModifierパターンの抽出
  - コンポーネント間の依存関係の最小化

5. レスポンシブ対応
  - Modifierでのブレイクポイント管理
  - コンテナクエリの活用
  - フレックスボックス/グリッドの適切な使用

# 出力形式
=============================================
1. 改善後のHTML
2. 各クラス名の説明とBEMルールとの対応
3. コンポーネント構造の解説'),

    ('prompt04', '04_開発テスト用',
    '# ここからがAIへの指示内容です
=============================================

# 以下のコードをレビューし、改善要件に基づいて修正してください。
-
# 入力コード部分
```
{code}
```
# 改善要件
=============================================
1. xx
  - xx
  - xx
  - xx

2. xx
  - xx
  - xx
  - xx

3. xx
  - xx
  - xx
  - xx

# 出力形式
=============================================
1. xx
2. xx
3. xx')
ON DUPLICATE KEY UPDATE 
    name = VALUES(name),
    content = VALUES(content);