// ========================================
// サーバーのメインファイル (index.js)
// VBAフォーマッターのバックエンド処理を担当
// ========================================

// 必要なモジュールのインポート
const path = require("path"); // pathモジュールを追加
const express = require("express");
const {analyzeCode} = require("./bedrock/analyzer"); // Bedrock AIの分析機能
// const { formatVBA } = require('./formatter'); // 基本的なフォーマット、現在機能使用していない
const db = require("./db/connection"); // データベース接続を追加

// Expressアプリケーションの初期化
const app = express();

// ミドルウェアの設定
// 静的ファイルのパスを修正- Docker環境での絶対パス指定に変更
app.use(express.static(path.join(__dirname, "public")));
// JSONリクエストの解析を有効化
app.use(express.json());

// CORSの設定を追加
app.use((req, res, next) => {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Methods", "GET, PUT, POST, DELETE");
  res.header("Access-Control-Allow-Headers", "Content-Type");
  next();
});

// Content-Typeの設定（APIエンドポイントのみに適用）
app.use((req, res, next) => {
  if (
    req.path.startsWith("/api") ||
    req.path.startsWith("/prompts") ||
    req.path === "/format" ||
    req.path === "/prompt-list"
  ) {
    res.setHeader("Content-Type", "application/json; charset=utf-8");
    // 追加: レスポンスエンコーディングを設定
    res.setHeader("Transfer-Encoding", "chunked");
  }
  next();
});

/**
 * プロンプト一覧取得APIをDB対応に修正
 * - MySQLからデータを取得するように変更
 */
app.get("/prompt-list", async (req, res) => {
  try {
    const [rows] = await db.query("SELECT id, name, description FROM prompts");
    res.json({
      success: true,
      prompts: rows,
    });
  } catch (error) {
    console.error("データベースエラー:", error);
    res.status(500).json({
      success: false,
      error: "プロンプト一覧の取得に失敗しました",
    });
  }
});

app.get("/about", (req, res) => {
  res.sendFile(path.join(__dirname, "public", "about.html"));
});

/**
 * 文字化け解消のための修正履歴 2024-03-xx
 * 
 * 1. Content-Typeヘッダーの設定を試行
 *    res.setHeader('Content-Type', 'application/json; charset=utf-8');
 *    → 部分的な効果あり、完全な解決には至らず
 *    
 * 2. Buffer経由のデコードを試行
 *    Buffer.from(rows[0].content).toString('utf8')
 *    → 一部の文字で文字化けが発生
 * 
 * 3. クエリでのCONVERT使用を試行
 *    SELECT CONVERT(content USING utf8mb4) as content
 *    SELECT CONVERT(CAST(content AS BINARY) USING utf8) as content
 *    → 特定の文字で文字化けが継続
 * 
 * 4. HEX形式での取得を試行
 *    SELECT HEX(content) as content_hex FROM prompts
 *    → デコード時に問題発生
 * 
 * 5. クエリオプションでのエンコーディング指定を試行
 *    {sql: "SELECT * FROM prompts", encoding: 'utf8mb4'}
 *    → mysql2では未サポート
 * 
 * 6. セッション文字コードの設定とバイナリ変換を試行
 *    SET NAMES utf8mb4 +
 *    SELECT CAST(CONVERT(content USING binary) AS CHAR CHARACTER SET utf8mb4)
 *    → 部分的な改善
 * 
 * 7. iconv-liteによる文字コード変換を試行
 *    const iconv = require('iconv-lite');
 *    iconv.decode(Buffer.from(content), 'utf8');
 *    → 特定のケースで例外発生
 * 
 * 8. execute()による実行を試行
 *    → db.executeは存在しない
 *    → プールインスタンスにはexecuteメソッドがない
 * 
 * 9. コネクション管理の改善を試行
 *    - getConnection()でコネクションを取得
 *    - 取得したコネクションでexecuteを実行
 *    - 処理後にコネクションを解放
 *    → コネクション管理は改善したが文字化けは解消せず
 * 
 * 10. 現在の解決策（2024-03-xx）：
 *     - データベースレベルでの文字コード設定の確実な適用
 *     - 接続時の初期化コマンドによる文字コード設定
 *     - バイナリ経由での確実な文字コード変換
 *     - レスポンスヘッダーとエンコーディングの最適化
 *     → 以下のアプローチの組み合わせで解決
 *        1. データベース接続時の文字コード初期化
 *        2. バイナリ経由での文字コード変換
 *        3. Buffer処理による確実なエンコーディング
 *        4. クライアントでのデコード処理の最適化
 */

/**
 * APIエンドポイントでの文字コード処理の詳細説明
 * 
 * 【文字化け解消の3段階】
 * 1. データベースからの取得時
 *    - バイナリデータとして一旦取得
 *    - UTF-8mb4として再解釈
 * 
 * 2. JSONレスポンス生成時
 *    - Buffer経由で文字コードを確実に変換
 *    - 文字化けしやすい文字も正しく処理
 * 
 * 3. クライアントへの送信時
 *    - Content-Typeヘッダーで文字コードを明示
 *    - Transfer-Encodingの設定で大きなデータも安全に送信
 * 
 * 【なぜこの方法で解決できたのか】
 * 1. バイナリ経由の変換
 *    - 文字コードの解釈を一旦リセット
 *    - 確実にUTF-8として再解釈
 * 
 * 2. 多層的なアプローチ
 *    - DB設定
 *    - 接続設定
 *    - アプリケーション処理
 *    全ての層で文字コードを適切に処理
 * 
 * 3. ヘッダー設定の最適化
 *    - クライアントに文字コードを正しく伝達
 *    - ブラウザでの解釈を確実に制御
 */

// プロンプト一覧を取得するエンドポイント
app.get("/api/prompt-info", async (req, res) => {
  try {
    // 文字コードの明示的な設定
    await db.query("SET NAMES utf8mb4");
    await db.query("SET CHARACTER SET utf8mb4");
    await db.query("SET SESSION collation_connection = utf8mb4_unicode_ci");
    
    // バイナリ経由での文字列取得（最新の解決策）
    const [rows] = await db.query(`
      SELECT 
        id,
        CONVERT(CAST(CONVERT(name USING binary) AS BINARY) USING utf8mb4) as name,
        CONVERT(CAST(CONVERT(content USING binary) AS BINARY) USING utf8mb4) as content
      FROM prompts 
      ORDER BY name
    `);
    
    // レスポンスヘッダーの設定
    res.setHeader('Content-Type', 'application/json; charset=utf8');
    res.setHeader('Transfer-Encoding', 'chunked');
    
    // Buffer経由でのエンコーディング
    const jsonString = JSON.stringify({
      success: true,
      prompts: rows.map(row => ({
        id: row.id,
        name: Buffer.from(row.name).toString('utf8'),
        content: Buffer.from(row.content).toString('utf8')
      }))
    });
    
    res.end(Buffer.from(jsonString, 'utf8'));

  } catch (error) {
    console.error("データ取得エラー:", error);
    res.status(500).json({
      success: false,
      error: "データの取得に失敗しました"
    });
  }
});

// 個別のプロンプト内容を取得するエンドポイント
app.get("/api/prompts/:id", async (req, res) => {
  // レスポンスヘッダーを設定
  res.setHeader('Content-Type', 'application/json; charset=utf-8');

  try {
    // UTF-8エンコーディングを明示的に指定してコンテンツを取得
    const [rows] = await db.query(
      "SELECT CAST(content AS CHAR CHARACTER SET utf8mb4) as content FROM prompts WHERE id = ?",
      [req.params.id]
    );

    if (!rows || rows.length === 0) {
      return res.status(404).json({
        success: false,
        error: "指定されたプロンプトが見つかりません"
      });
    }

    // 成功時のレスポンス形式を統一
    res.json({
      success: true,
      content: rows[0].content
    });

  } catch (error) {
    // エラーログの出力
    console.error("プロンプト取得エラー:", error);
    
    // エラー時のレスポンス形式を統一
    res.status(500).json({
      success: false,
      error: "プロンプトの取得に失敗しました",
      details: process.env.NODE_ENV === 'development' ? error.message : undefined
    });
  }
});

/**
 * フォーマットAPIにプロンプトIDの処理を追加
 * - リクエストからプロンプトIDを取得
 * - DBから対応するプロンプトを取得して使用
 */
app.post("/format", async (req, res) => {
  try {
    const {code, promptId} = req.body;
    const [rows] = await db.query(
      "SELECT CAST(content AS CHAR CHARACTER SET utf8mb4) as content FROM prompts WHERE id = ?",
      [promptId]
    );

    if (!rows || rows.length === 0) {
      return res.status(404).json({
        success: false,
        error: "プロンプトが見つかりません",
      });
    }

    const formattedCode = await analyzeCode(code, rows[0].content);
    res.json({success: true, formatted: formattedCode});
  } catch (error) {
    console.error("Format error:", error);
    res.status(500).json({
      success: false,
      error: error.message || "フォーマット処理に失敗しました",
    });
  }
});

// プロンプト更新API
app.put("/prompts/:id", async (req, res) => {
  console.log("Update request received:", {
    params: req.params,
    body: req.body,
  });

  try {
    const {id} = req.params;
    const {content} = req.body;

    // データベース接続テスト
    const [testConnection] = await db.query("SELECT 1");
    console.log("Database connection test:", testConnection);

    const [result] = await db.query(
      "UPDATE prompts SET content = ? WHERE id = ?",
      [content, id]
    );

    console.log("Update result:", result);

    if (result.affectedRows === 0) {
      return res.status(404).json({
        success: false,
        error: "プロンプトが見つかりません",
      });
    }

    res.json({
      success: true,
      message: "更新しました",
    });
  } catch (error) {
    console.error("Database error:", error);
    res.status(500).json({
      success: false,
      error: error.message,
    });
  }
});

// サーバー起動
const PORT = process.env.PORT || 4000;
app.listen(PORT, "0.0.0.0", () => {
  console.log("========================================");
  console.log(`Server running on http://localhost:${PORT}`);

  // 登録されているルートを表示
  app._router.stack.forEach((r) => {
    if (r.route && r.route.path) {
      console.log(`Route: ${r.route.path}`);
      console.log(`Methods:`, r.route.methods);
    }
  });
  console.log("========================================");
});
