/**
 * ページ初期ロード時の処理
 * 
 * 【プロンプト選択の仕組み】
 * 1. ページロード時にプロンプト一覧を取得
 * 2. セレクトボックスの最初のオプションが自動的に選択される
 * 3. changePrompt()が呼ばれ、選択されたプロンプトの内容を表示
 * 
 * 【選択の優先順位】
 * 1. セレクトボックスの最初のオプション（データベースのORDER BY name順で最初のプロンプト）
 * 2. エラー時は「プロンプトの読み込みに失敗しました」というメッセージを表示
 */
window.onload = async function() {
    try {
        // プロンプトリストの取得
        const response = await fetch('/api/prompt-info');
        const buffer = await response.arrayBuffer();
        const decoder = new TextDecoder('utf-8');
        const jsonString = decoder.decode(buffer);
        const data = JSON.parse(jsonString);
        
        if (data.success) {
            // セレクトボックスの生成
            // ORDER BY name でソートされたプロンプトリストの最初のものが自動選択される
            const select = document.getElementById('promptSelect');
            select.innerHTML = data.prompts.map(prompt => 
                `<option value="${prompt.id}">${prompt.name}</option>`
            ).join('');

            // 最初のプロンプトの内容を取得して表示
            await changePrompt();
        }
    } catch (error) {
        console.error('初期化エラー:', error);
        document.querySelector('.prompt-display').textContent = 'プロンプトの読み込みに失敗しました';
    }
};

// プロンプト切り替え関数
async function changePrompt() {
    try {
        const selectedPrompt = document.getElementById('promptSelect').value;
        console.log('Selected prompt ID:', selectedPrompt); // デバッグログ

        const response = await fetch(`/api/prompts/${selectedPrompt}`);
        const data = await response.json();
        console.log('API Response:', data); // デバッグログ
        
        const promptDisplay = document.querySelector('.prompt-display');
        if (data.success) {
            promptDisplay.textContent = data.content;
        } else {
            promptDisplay.textContent = 'プロンプトの読み込みに失敗しました';
        }
    } catch (error) {
        console.error('プロンプト取得エラー:', error);
        document.querySelector('.prompt-display').textContent = 'プロンプトの読み込みに失敗しました';
    }
}

/**
 * コードのフォーマットを実行する関数
 * 1. 入力を取得
 * 2. サーバーにリクエスト
 * 3. 結果を表示
 */
async function formatCode() {
    const input = document.getElementById('input').value;
    const promptId = document.getElementById('promptSelect').value;
    const formatButton = document.getElementById('formatButton');
    
    // デバッグログ追加
    console.log('Formatting with:', {
        promptId,
        code: input
    });

    if (!input) {
        alert('コードを入力してください');
        return;
    }

    formatButton.innerHTML = '<span class="processing"><span></span><span></span><span></span></span>';

    try {
        const response = await fetch('/format', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ 
                code: input,
                promptId: promptId
            })
        });

        // デバッグログ追加
        console.log('Response status:', response.status);
        const data = await response.json();
        console.log('API Response:', data);
        
        if (data.success) {
            document.getElementById('output').value = data.formatted;
            formatButton.innerHTML = '分析完了! クリックしてコピー<i class="fa-regular fa-copy fa-beat-fade fa-lg" style="vertical-align: 0;"></i>';
            formatButton.onclick = copyToClipboard;
        } else {
            formatButton.textContent = 'エラー: ' + data.error;
        }
    } catch (error) {
        console.error('フォーマットエラー:', error);
        formatButton.textContent = 'エラー: ' + error.message;
    }
    formatButton.classList.remove('processing');
}

/**
 * クリップボードにコピーする関数
 * 出力用テキストエリアの内容をクリップボードにコピー
 */
 function copyToClipboard() {
    const outputTextarea = document.getElementById('output');
    outputTextarea.select();
    document.execCommand('copy');

    // ポップアップメッセージを表示
    const popup = document.getElementById('popup');
    popup.classList.add('show');

    // 一定時間後にポップアップメッセージを非表示にする
    setTimeout(() => {
        popup.classList.remove('show');
    }, 2000); // 2秒後に非表示
}

/**
 * ファイル選択時の処理
 * 選択されたファイルの内容を入力エリアに表示
 */
document.getElementById('vbaFile').addEventListener('change', (e) => {
    // 選択されたファイルを取得
    const file = e.target.files[0];
    if (file) {
        // FileReaderを使用してファイルの内容を読み込み
        const reader = new FileReader();
        // ファイル読み込み完了時の処理
        reader.onload = (e) => {
            // 読み込んだ内容を入力エリアに設定
            document.getElementById('input').value = e.target.result;
        };
        // ファイルをテキストとして読み込み開始
        reader.readAsText(file);
    }
});

// プロンプトの保存
async function savePrompt() {
    try {
        const promptId = document.getElementById('promptSelect').value;
        const content = document.getElementById('promptDisplay').textContent;
        
        console.log('Saving prompt:', promptId);
        console.log('Content:', content);

        const saveButton = document.getElementById('saveButton');
        saveButton.textContent = '保存中...';
        saveButton.disabled = true;

        // URLを確認のため出力
        const url = `/prompts/${promptId}`;
        console.log('Request URL:', url);

        const response = await fetch(url, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                content: content
            })
        });

        console.log('Response status:', response.status);
        const data = await response.json();
        console.log('Response data:', data);
        
        if (data.success) {
            alert('保存しました');
            document.getElementById('editButton').click();
        } else {
            alert('保存に失敗しました: ' + data.error);
        }

    } catch (error) {
        console.error('保存エラー:', error);
        alert('エラーが発生しました: ' + error.message);
    } finally {
        const saveButton = document.getElementById('saveButton');
        saveButton.textContent = '保存';
        saveButton.disabled = false;
    }
}

// 編集モードの切り替え
function toggleEdit() {
    const promptDisplay = document.getElementById('promptDisplay');
    const editButton = document.getElementById('editButton');
    const saveButton = document.getElementById('saveButton');
    const isEditing = promptDisplay.contentEditable === 'true';

    if (isEditing) {
        // 編集モード終了
        promptDisplay.contentEditable = 'false';
        editButton.textContent = '編集';
        saveButton.style.display = 'none';
        promptDisplay.classList.remove('editing');
    } else {
        // 編集モード開始
        promptDisplay.contentEditable = 'true';
        editButton.textContent = 'キャンセル';
        saveButton.style.display = 'block';
        promptDisplay.classList.add('editing');
        
        // カーソルを末尾に設定
        const range = document.createRange();
        const sel = window.getSelection();
        range.selectNodeContents(promptDisplay);
        range.collapse(false);
        sel.removeAllRanges();
        sel.addRange(range);
        promptDisplay.focus();
    }
}

// データ取得時の文字コード処理
async function fetchPromptInfo() {
  try {
    const response = await fetch('/api/prompt-info');
    const buffer = await response.arrayBuffer();
    const decoder = new TextDecoder('utf-8');
    const jsonString = decoder.decode(buffer);
    return JSON.parse(jsonString);
  } catch (error) {
    console.error('データ取得エラー:', error);
    throw error;
  }
}